@@ -1,3 +1,61 @@
+package Term::Screen::Win32::CursorAndSize;
+use 5.005;
+use strict;
+use warnings;
+
+use Carp;
+use Win32::Console::ANSI;
+
+use Tie::Hash;
+our @ISA = ('Tie::Hash');
+
+$|++;
+
+sub TIEHASH
+	{
+	my $storage = bless {}, $_[0];
+	return $storage;
+	}
+
+sub STORE
+	{
+	my $key = lc($_[1]);
+
+	if ($key eq 'c')
+		{ printf("\e[%d;%dH", (Win32::Console::ANSI::Cursor())[1], $_[2]+1); }
+	elsif ($key eq 'r')
+		{ printf("\e[%d;%dH", $_[2]+1, (Win32::Console::ANSI::Cursor())[0]); }
+	elsif ($key eq 'cols')
+		{
+		if (!Win32::Console::ANSI::SetConsoleSize($_[2], (Win32::Console::ANSI::XYMax())[1]))
+			{ croak 'Could not set console size: '.$^E; };
+		}
+	elsif ($key eq 'rows')
+		{
+		if (!Win32::Console::ANSI::SetConsoleSize((Win32::Console::ANSI::XYMax())[0], $_[2]))
+			{ croak 'Could not set console size: '.$^E; };
+		}
+	else
+		{ $_[0]{$_[1]} = $_[2]; };
+	};
+
+sub FETCH
+	{
+	my $key = lc($_[1]);
+
+	if ($key eq 'c')
+		{ return ((Win32::Console::ANSI::Cursor())[0] - 1); }
+	elsif ($key eq 'r')
+		{ return ((Win32::Console::ANSI::Cursor())[1] - 1); }
+	elsif ($key eq 'cols')
+		{ return (Win32::Console::ANSI::XYMax())[0]; }
+	elsif ($key eq 'rows')
+		{ return (Win32::Console::ANSI::XYMax())[1]; }
+	else
+		{ return $_[0]{$_[1]}; };
+	};
+
+
 package Term::Screen::Win32;
 
 use 5.005;
@@ -29,7 +87,7 @@ our @EXPORT = qw(
 	
 );
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 
 # Preloaded methods go here.
@@ -38,33 +96,32 @@ sub term
 	{ croak 'This function is not supported on your platform ('.$^O.')'; };
 
 sub rows
-	{ return (resize($_[0], undef, $_[1]))[1]; };
+	{ return $_[0]->{'rows'}; };
 
 sub cols
-	{ return (resize($_[0], $_[1], undef))[0]; };
+	{ return $_[0]->{'cols'}; };
 
 sub at
-	{ return Win32::Console::ANSI::Cursor((defined($_[0]) ? $_[0]+1 : 0), (defined($_[1]) ? $_[1]+1 : 0)); };
+	{
+	if (defined($_[1])) { $_[0]->{'r'} = $_[1]; };
+	if (defined($_[2])) { $_[0]->{'c'} = $_[2]; };
+	return $_[0];
+	};
 
 sub resize
 	{
-	my ($curCols, $curRows) = (Win32::Console::ANSI::XYMax())[0, 1];
-
-	if (defined($_[1]) || defined($_[2]))
-		{
-		if (!SetConsoleSize((defined($_[1]) ? $_[1] : $curCols), (defined($_[2]) ? $_[2] : $curRows)))
-			{ croak 'Could not set console size: '.$^E; };
-		};
+	if (defined($_[1])) { $_[0]->{'rows'} = $_[1]; };
+	if (defined($_[2])) { $_[0]->{'cols'} = $_[2]; };
 	
-	return (Win32::Console::ANSI::XYMax())[0, 1];
+	return ($_[0]->{'rows'}, $_[0]->{'cols'});
 	};
 
 sub normal      { print "\e[0m"; return $_[0]; };
 sub bold        { print "\e[1m"; return $_[0]; };
 sub reverse     { print "\e[7m"; return $_[0]; };
-sub clrscr      { Cls();         return $_[0]; };
+sub clrscr      { print "\e[2J"; return $_[0]; };
 sub clreol      { print "\e[0K"; return $_[0]; };
-sub clreos      { print "\e[1J"; return $_[0]; };
+sub clreos      { print "\e[0J"; return $_[0]; };
 sub il          { print "\e[".(defined($_[1]) ? $_[1] : 1).'L'; return $_[0]; };
 sub dl          { print "\e[".(defined($_[1]) ? $_[1] : 1).'M'; return $_[0]; };
 sub ic_exists   { return 1; };
@@ -75,8 +132,8 @@ sub puts        { my $this = shift; print(@_); return $this; };
 
 sub getch
 	{
-	if (scalar(@{$_[0]->{'key_pressed'}}) || key_pressed($_[0], 0))
-		{ return shift(@{$_[0]->{'key_pressed'}}); };
+	key_pressed($_[0], 0);
+	return shift(@{$_[0]->{'key_pressed'}});
 	};
 
 sub def_key
@@ -93,22 +150,37 @@ sub parseKeyEvent
 	return 'noop';
 	};
 
+sub fetchKeyEvent
+	{
+	while($_[0]->{'console'}->GetEvents())
+		{
+		my @key_pressed = $_[0]->{'console'}->Input();
+		if (defined($key_pressed[0]) && ($key_pressed[0] == 1) && $key_pressed[1])
+			{ return \@key_pressed; };
+		};
+	return undef;
+	};
+
 sub key_pressed
 	{
 	if (scalar(@{$_[0]->{'key_pressed'}}))
 		{ return 1; };
 
 	my $expTime = time() + (defined($_[1]) ? (($_[1] > 0) ? $_[1] : 999999) : -1);
-	while($_[0]->{'console'}->GetEvents())
+
+	while(1)
 		{
-		my @key_pressed = $_[0]->{'console'}->Input();
-		if (defined($key_pressed[0]) && ($key_pressed[0] == 1) && $key_pressed[1])
+		my $keyEvent = fetchKeyEvent($_[0]);
+
+		if (defined($keyEvent))
 			{
-			push(@{$_[0]->{'key_pressed'}}, parseKeyEvent($_[0], \@key_pressed));
+			push(@{$_[0]->{'key_pressed'}}, parseKeyEvent($_[0], $keyEvent));
 			return 1;
 			};
+
 		if (time() > $expTime)
 			{ last; };
+		
 		sleep(0.02);
 		};
 	return 0;
@@ -165,10 +237,13 @@ sub new($%)
 	{
 	my ($class) = @_;
 
-	my $self = {'key_pressed' => [],
-	            'def_key'     => {},
-	            'console'     => Win32::Console->new(STD_INPUT_HANDLE),
-               };
+	my $self = undef;
+
+	tie(%{$self}, 'Term::Screen::Win32::CursorAndSize');
+
+	$self->{'key_pressed'} = [],
+	$self->{'def_key'}     = {},
+	$self->{'console'}     = Win32::Console->new(STD_INPUT_HANDLE),
 
     $self->{'origMode'} = $self->{'console'}->Mode();
     $self->{'console'}->Mode(ENABLE_PROCESSED_INPUT);
@@ -182,9 +257,13 @@ sub new($%)
 sub cleanup
 	{
 	$_[0]->normal();
-	$_[0]->{'console'}->Mode($_[0]->{'origMode'});
+	if (defined($_[0]->{'console'}))
+		{ $_[0]->{'console'}->Mode($_[0]->{'origMode'}); };
 	};
 
+sub DESTROY
+	{ cleanup(@_); };
+
 
  1;
 __END__
@@ -194,7 +273,7 @@ __END__
 
 Term::Screen::Win32 - Simple L<Term::Screen> style interface to the L<Win32::Console> (and L<Win32::Console::ANSI>) capabilities
 
-I<Version 0.02>
+I<Version 0.03>
 
 =head1 SYNOPSIS
 
@@ -225,6 +304,12 @@ These functions are different from L<Term::Screen>:
 
 Provide 'virtual keycode' as an 'input string'
 
+These functions are not exists in L<Term::Screen>:
+
+=item C<cleanup()>
+
+Return console to the origuinal mode. Called automatically on C<DESTROY>.
+
 =head2 EXPORT
 
 None.
